/*
 * deviced
 *
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <string.h>
#include <vconf.h>

#include "core/log.h"
#include "core/common.h"
#include "core/config-parser.h"
#include "core/launch.h"
#include "core/device-notifier.h"
#include "usb.h"

static bool debug_state;

bool usb_debug_state(void)
{
	return debug_state;
}

void set_usb_debug_state(bool on)
{
	int state = on ? 1 : 0;

	if (vconf_set_bool(VCONFKEY_SETAPPL_USB_DEBUG_MODE_BOOL, state) == 0)
		debug_state = on;
}

static void usb_debug_changed(keynode_t *key, void *data)
{
	int mode;

	if (!key)
		return;

	mode = vconf_keynode_get_bool(key);

	if (mode == debug_state)
		return;

	debug_state = mode;

	_I("USB debug %s", mode ? "ON" : "OFF");

	device_notify(DEVICE_NOTIFIER_USB_DEBUG_MODE, (void *)&mode);
}

static int usb_debug_mode_changed(void *data)
{
	int on;
	unsigned mode;
	int ret;

	on = *(int *)data;

	mode = usb_state_get_current_mode();
	if (mode == 0)
		mode = usb_state_get_selected_mode();

	_I("Previous mode (%d)", mode);

	if (on)
		mode |= USB_FUNCTION_SDB;
	else
		mode &= ~USB_FUNCTION_SDB;

	_I("New mode (%d)", mode);

	ret = usb_change_mode(mode);
	if (ret < 0)
		_E("Failed to change usb mode to (%d)", mode);

	return ret;
}

void add_usb_debug_handler(void)
{
	int state = 0;

	if (vconf_notify_key_changed(VCONFKEY_SETAPPL_USB_DEBUG_MODE_BOOL,
				usb_debug_changed, NULL) != 0)
		_E("Failed to add usb debug handler");

	register_notifier(DEVICE_NOTIFIER_USB_DEBUG_MODE,
			usb_debug_mode_changed);

	if (vconf_get_bool(VCONFKEY_SETAPPL_USB_DEBUG_MODE_BOOL, &state) == 0)
		debug_state = (state == 0 ? false : true);
}

void remove_usb_debug_handler(void)
{
	vconf_ignore_key_changed(VCONFKEY_SETAPPL_USB_DEBUG_MODE_BOOL,
			usb_debug_changed);

	unregister_notifier(DEVICE_NOTIFIER_USB_DEBUG_MODE,
			usb_debug_mode_changed);
}
